<?php

namespace XF2SEO\SEOPackage;

use XF2SEO\SEOPackage\Entity\Redirect;

class Listener
{
    public static function appSetup(\XF\App $app): void
    {
        $baseControllerPath = \XF::getRootDirectory() . '/src/XF/Mvc/Controller.php';
        $contents = file_get_contents($baseControllerPath);
        if ($contents === false) {
            return;
        }
        $contents = ltrim($contents);
        $contents = substr($contents, 5); // remove <?php
        $contents = str_replace(
            'abstract class Controller',
            'abstract class _Controller',
            $contents
        );

        $extends = '
            class Controller extends _Controller {
                public function assertCanonicalUrl($linkUrl)
                {
                    $targetUrl = \XF2SEO\SEOPackage\Listener::getRedirectTarget($linkUrl);
                    if ($targetUrl !== null) {
                        return;
                    }
                    
                    parent::assertCanonicalUrl($linkUrl);
                }
            }
        ';
        eval($contents . PHP_EOL . $extends);
    }

    public static function getRedirectTarget(string $link): ?Redirect
    {
        $redirects = \XF::finder('XF2SEO\SEOPackage:Redirect')
            ->where('active', 1)
            ->fetch();
        if ($redirects->count() === 0) {
            return null;
        }

        /** @var Redirect $redirect */
        foreach ($redirects as $redirect) {
            if (stripos($redirect->match_url, $link) === 0
                && utf8_strlen($redirect->match_url) === utf8_strlen($link)
            ) {
                return $redirect;
            }
        }

        return null;
    }

    public static function dispatcherMatch(\XF\Mvc\Dispatcher $dispatcher, \XF\Mvc\RouteMatch &$match): void
    {
        $requestUri = \XF::app()->request()->getRequestUri();
        $redirect = self::getRedirectTarget($requestUri);

        if ($redirect !== null) {
            header("Cache-Control: no-store, no-cache, must-revalidate, max-age=0");
            header("Cache-Control: post-check=0, pre-check=0", false);
            header("Pragma: no-cache");
            header(
                'Location: ' . \XF::canonicalizeUrl($redirect->target_url),
                true,
                $redirect->getRedirectStatus()
            );

            exit();
        }
    }
}
